/*Copyright (c) 2015-2050, JD All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License. */

/* --------------------------------------------------
 * @brief: 头文件 一个设备示例
 *
 * @version: 1.0
 *
 * --------------------------------------------------
 */

#ifndef _JOYLINK_DEVICE_DEMO_H_
#define _JOYLINK_DEVICE_DEMO_H_

#include <stdio.h>
#include "joylink.h"
#include "joylink_protocol.h"
#include "joylink_sdk.h"

#include "joylink_stdint.h"
#include "joylink_time.h"


#ifdef __cplusplus
extern "C"{
#endif /* __cplusplus */

/**
 * @brief  宏定义
 * 
 */
#define SDK_CONFIG_HOST_PROTOCOL    ("mqtt_tls")
//#define SDK_CONFIG_HOST_NAME        ("116.196.102.221")
#define SDK_CONFIG_HOST_NAME        ("Please complete in hostname")
//#define SDK_CONFIG_HOST_PORT        (32222)
#define SDK_CONFIG_HOST_PORT        ()
//#define SDK_CONFIG_DEVICE_ID        ("207190dd7a13e00")
#define SDK_CONFIG_DEVICE_ID        ("Please complete in deviceId")
#define SDK_CONFIG_CA_PATH          ("example/certs/ca.crt")
#define SDK_CONFIG_CERT_PATH        ("example/certs/2cbcb2d9c1ec234.crt")
#define SDK_CONFIG_KEY_PATH         ("example/certs/2cbcb2d9c1ec234.key")

#define DEVICE_INFO_THING_MODEL_ID        ("urn:user-spec-v1:thing-model:RSU_LD:8786247f5d335d3cb994d149e6cf61bd")
#define DEVICE_INFO_THING_MODEL_VERSION   ("v0.0.1")

#define DEVICE_INFO_UUID                  ("123456")
#define DEVICE_INFO_SHARED_KEY            ("1234567890123456")

/**
 * @brief: 此函数需返回设备的MAC地址
 *
 * @param[out] out: 将设备MAC地址赋值给此参数
 * 
 * @returns: E_RET_OK 成功, E_RET_ERROR 失败
 */
int32_t
joylink_dev_get_user_mac(char *out);

/**
 * @brief  设备初始化
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 */
int32_t device_initialise(struct joylink_sdk_t *sdk);

/**
 * @brief  设备反初始化
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 */
int32_t device_deinitialise(struct joylink_sdk_t *sdk);

/**
 * @brief  连接成功 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk连接京东云成功后，包括初次连接以及其它原因导致的断网重连，都会调用该函数
 */
int32_t device_callback_connect(struct joylink_sdk_t *sdk);

/**
 * @brief  断开连接 回调函数
 *
 * @param[in] sdk: joylink sdk 结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk断开连接，包括主动断开，由其它原因导致的被动断开，都会调用该函数
 */
int32_t device_callback_disconnect(struct joylink_sdk_t *sdk);

/**
 * @brief  物模型上报响应 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] service_key: 服务实例名字
 * @param[in] in_res: 物模型上报响应消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 进行无模型上报后，收到上报响应后 会调用该函数
 */
int32_t device_callback_thingmodel_post_response(struct joylink_sdk_t *sdk,
                        char *obj_name, char *service_key, JoyProtoThingModelPostRes_t *in_res);

/**
 * @brief  属性设置 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] service_key: 服务实例名字
 * @param[in] in_set: 属性设置消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 收到属性设置消息时 会调用该函数
 */
int32_t device_callback_property_set(struct joylink_sdk_t *sdk,
                        char *obj_name, char *service_key, JoyProtoPropSet_t *in_set);

/**
 * @brief  属性获取 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] service_key: 服务实例名字
 * @param[in] in_get: 属性获取消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 收到属性设置消息时 会调用该函数
 */
int32_t device_callback_property_get(struct joylink_sdk_t *sdk,
                        char *obj_name, char *service_key, JoyProtoPropGet_t *in_get);

/**
 * @brief  方法调用 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] service_key: 服务实例名字
 * @param[in] in_call: 方法调用消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 收到方法调用消息时 会调用该函数
 */
int32_t device_callback_function_call(struct joylink_sdk_t *sdk,
                        char *obj_name, char *service_key, JoyProtoFuncCall_t *in_call);

/**
 * @brief  自动注册响应 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] service_key: 服务实例名字
 * @param[in] in_call: 方法调用消息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 自动注册响应 会调用该函数
 */
int32_t device_callback_register_response(struct joylink_sdk_t *sdk,
                        char *obj_name, char *service_key, JoyProtoRegReqRes_t *in_res);

/**
 * @brief  NTP请求响应 回调函数
 * 
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] set_time: 设置时间的结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 收到NTP请求响应 会调用该函数
 */
int32_t device_callback_ntp_response(struct joylink_sdk_t *sdk, jl_time_stamp_t *set_time);

/**
 * @brief  ntp设置 回调函数
 *
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] service_key: 服务实例名字
 * @param[in] in_set: ntp设置消息结构体指针
 * @return
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 收到ntp设置消息时 会调用该函数
 */
int32_t device_callback_ntp_set(struct joylink_sdk_t *sdk,
                        char *obj_name, char *service_key, JoyProtoNtpSet_t *in_set);

/**
 * @brief  连接地址分发 回调函数
 *
 * @param[in] sdk: joylink sdk 结构体指针
 * @param[in] obj_name: 对象空间，可选值{"device", "edge", "group"}
 * @param[in] service_key: 服务实例名字
 * @param[in] in_set: 连接地址分发消息结构体指针
 * @return
 *    返回值 JoylinkRetCode_E
 * @note
 *    sdk 收到连接地址分发消息时 会调用该函数
 */
int32_t device_callback_linkaddress_reissue(struct joylink_sdk_t *sdk,
                        char *obj_name, char *service_key, JoyProtoLinkReissue_t *in_set);

/**
 * @brief  设备任务线程函数
 * 
 * @note
 *    设备自己的任务线程
 */
void device_task_thread(void);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _DEVICE_DEMO_H_ */
