/*Copyright (c) 2015-2050, JD All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License. */

/* --------------------------------------------------
 * @brief: joylink sdk 使用示例程序
 *
 * @version: 1.0
 *
 * --------------------------------------------------
 */

#include <stdio.h>
#include "joylink.h"
#include "joylink_sdk.h"
#include "joylink_protocol.h"

#include "joylink_log.h"
#include "joylink_string.h"
#include "joylink_memory.h"
#include "joylink_stdint.h"

#include "joylink_device_demo.h"
#include "cJSON.h"

#include "joylink_softap.h"
#include "joylink_softap_start.h"

#include "joylink_ble_sdk.h"

extern int joylink_util_hexStr2bytes(const char *hexStr, uint8_t *buf, int bufLen);

void joylink_ble_init(void)
{
    char mac_str[17] = {0};
    jl_dev_info_t info;

    jl_platform_memset(&info, 0, sizeof(jl_dev_info_t));
    jl_platform_strcpy(info.product_uuid, DEVICE_INFO_UUID);
    jl_platform_strcpy(info.shared_key, DEVICE_INFO_SHARED_KEY);
    joylink_dev_get_user_mac(mac_str);
    joylink_util_hexStr2bytes(mac_str, info.mac, 6);

    jl_init(&info);
}

int32_t joylink_sdk_start_up(void)
{
    while (1) {
        /***********************************************************************
         * 测试函数
         ***********************************************************************/
        joylink_sdk_test();
        
        /***********************************************************************
         * joylink sdk 配置参数
         ***********************************************************************/
        joylink_sdk_config_t *config = joylink_sdk_config_create();
        if (config == NULL) {
            log_error("joylink sdk config create failed");
            return -1;
        }
        jl_platform_memset(config, 0, sizeof(joylink_sdk_config_t));

        // linkaddress 协议
        joylink_linkaddress_protocol_get(config->protocol);
        if (config->protocol && jl_platform_strlen(config->protocol)) {
            log_info("joylink_linkaddress_protocol_get success");
        } else {
            jl_platform_strcpy(config->protocol, SDK_CONFIG_HOST_PROTOCOL);
            joylink_linkaddress_protocol_set(config->protocol);
            log_info("joylink_linkaddress_protocol_get failed, use default protocol:%s", config->protocol);
        }

        // linkaddress 主机地址
        config->hostname = (char *) jl_platform_malloc(64 * sizeof(char));
        if (config->hostname != NULL) {
            jl_platform_memset(config->hostname, 0, 64 * sizeof(char));
        } else {
            log_error("malloc config->hostname failed");
        }
        joylink_linkaddress_hostname_get(config->hostname);
        if (config->hostname && jl_platform_strlen(config->hostname)) {
            log_info("joylink_linkaddress_hostname_get success");
        } else {
            jl_platform_strcpy(config->hostname, SDK_CONFIG_HOST_NAME);
            joylink_linkaddress_hostname_set(config->hostname);
            log_info("joylink_linkaddress_hostname_get failed, use default hostname:%s", config->hostname);
        }

        // linkaddress 端口号
        joylink_linkaddress_port_get(&config->port);
        if (config->port > 0) {
            log_info("joylink_linkaddress_port_get success");
        } else {
            config->port = SDK_CONFIG_HOST_PORT;
            joylink_linkaddress_port_set(config->port);
            log_info("joylink_linkaddress_port_get failed, use default port:%d", config->port);
        }

        // 设备id，用户名和密码
        config->deviceId = SDK_CONFIG_DEVICE_ID;
        config->username = SDK_CONFIG_DEVICE_ID;

        // TLS 证书
        config->cafile = SDK_CONFIG_CA_PATH;
        config->cert = SDK_CONFIG_CERT_PATH;
        config->key = SDK_CONFIG_KEY_PATH;

        // 验证主机名
        config->insecure = true;
        config->is_hold_loop = true;
        config->is_gzip_payload = true;
        log_debug("protocol = %s\nhostname = %s\nport = %d\ndeviceId = %s\nusername = %s\ncafile = %s\ncert = %s\nkey = %s\ninsecure = %d", 
            config->protocol,
            config->hostname, 
            config->port, 
            config->deviceId, 
            config->username, 
            config->cafile, 
            config->cert, 
            config->key,
            config->insecure
        );

        /***********************************************************************
         * joylink sdk 创建
         ***********************************************************************/
        struct joylink_sdk_t *joy_sdk = joylink_sdk_create(config);

        /***********************************************************************
         * joylink sdk 初始化
         ***********************************************************************/
        joylink_sdk_initialise(joy_sdk);

        /***********************************************************************
         * joylink sdk 设置回调函数
         ***********************************************************************/
        // 设置 连接成功回调函数
        joylink_sdk_set_dev_cb_connect(joy_sdk, device_callback_connect);

        // 设置 连接断开回调函数
        joylink_sdk_set_dev_cb_disconnect(joy_sdk, device_callback_disconnect);

        // 设置 物模型上报响应回调函数
        joylink_sdk_set_dev_cb_thmd_post_res(joy_sdk, device_callback_thingmodel_post_response);

        // 设置 属性设置回调函数
        joylink_sdk_set_dev_cb_prop_set(joy_sdk, device_callback_property_set);

        // 设置 属性获取回调函数
        joylink_sdk_set_dev_cb_prop_get(joy_sdk, device_callback_property_get);

        // 设置 方法调用回调函数
        joylink_sdk_set_dev_cb_func_call(joy_sdk, device_callback_function_call);

        // 设置 自动注册响应回调函数
        joylink_sdk_set_dev_cb_reg_res(joy_sdk, device_callback_register_response);

        // 设置 ntp请求回调函数
        joylink_sdk_set_dev_ntp_req_res(joy_sdk, device_callback_ntp_response);

        // 设置 ntp设置回调函数
        joylink_sdk_set_dev_cb_ntp_set(joy_sdk, device_callback_ntp_set);

        // 设置 设备连接地址分发回调函数
        joylink_sdk_set_dev_cb_linkaddress_reissue(joy_sdk, device_callback_linkaddress_reissue);

        /***********************************************************************
         * 设备 初始化
         ***********************************************************************/
        device_initialise(joy_sdk);

        /***********************************************************************
         * joylink sdk 连接京物联平台, 连接五次失败，则切换地址重新连接
         ***********************************************************************/
        int times = 5, ret = JOYLINK_RET_FAILED;
        do {
            ret = joylink_sdk_connect(joy_sdk);
            if (ret) {
                log_error("joylink_sdk_connect failed, ret:%d", ret);
                times--;
                if (!times) {
                    joylink_linkaddress_exchange();
                    goto RET;
                }
            } else {
                log_info("joylink_sdk_connect success, ret:%d", ret);
            }
        } while (ret);
        
        /***********************************************************************
         * joylink sdk 进入主循环
         ***********************************************************************/
        joylink_sdk_main_loop(joy_sdk);

    RET:
        /***********************************************************************
         * 设备 反初始化
         ***********************************************************************/
        device_deinitialise(joy_sdk);

        /***********************************************************************
         * joylink sdk 销毁 释放资源
         ***********************************************************************/
        joylink_sdk_destory(joy_sdk);

        /***********************************************************************
         * joylink sdk 配置 销毁 释放资源
         ***********************************************************************/
        jl_platform_memset(config, 0, sizeof(joylink_sdk_config_t));
        joylink_sdk_config_destory(config);
    }
    return 0;
}

/**
 * @brief  主函数
 * 
 */
int main()
{
    /***********************************************************************
     * SoftAP 配网
     ***********************************************************************/
    // 进入AP模式
    // jl_softap_enter_ap_mode();

    // 开始softAP 配网
    // joylink_softap_start(5 * 100);
    
    //初始化BLE
    // joylink_ble_init();

    /***********************************************************************
     * 时间同步
     ***********************************************************************/
    // TODO 如果开发者有自己的时间同步机制，可在此执行，如果没有自己的同步机制，可参考示例
    // 使用SDK提供的API来进行时间的校准
    joylink_sdk_start_up();
    return 0;
}

// end of file
