/*************************************

Copyright (c) 2015-2050, JD Smart All rights reserved. 

*************************************/


#ifndef _JOYLINK_SOFTAP_PROTOCAL_H
#define _JOYLINK_SOFTAP_PROTOCAL_H
#include "joylink_softap_util.h"

#ifdef __cplusplus
extern "C"{
#endif /* __cplusplus */

// softAP ssid
extern uint8 softap_ssid[MAX_LEN_OF_SSID+1];

/**
 * @brief: softAP流程初始化
 * 
 * @return: Always TRUE
*/
int joylink_softap_init(void);

/**
 * @brief: 处理TCP socket的数据
 * 
 * @return: 
*/
int joylink_softap_data_packet_handle(int socket_fd, uint8 *msg, int16 count);	

/**
 * @brief: UDP广播local public key
 * 
 * @param[in] socket_fd: UDP socket句柄
 * 
 * @result: 1 成功, 0 失败
*/
int joylink_softap_udpbroad(int socket_fd);			

/**
 * @brief: 获取softAP配网状态,如果配网成功则返回joylinkSoftAP_Result_t其中包含wifi路由器的ssid和password
 * 
 * @param[out] pRet: softAP配网结果,如果配网成功则包含正确的wifi路由器ssid和password
 * 
 * @result: 0 配网成功, -1 失败
 * 
*/
int joylink_softap_result(joylinkSoftAP_Result_t* pRet);

/**
 * @brief: 发送udp广播包 
 *
 * @param[in] socket_fd: socket句柄
 * @param[in] buf: 要广播的数据
 * @param[in] len: 数据长度
 *
 * @returns:   实际发送数据的长度, -1 发送失败
 */
int joylink_udp_broad_send(int socket_fd, char *buf, int len);

/**
 * @brief: 配网过程中所有的socket send操作
 *
 * @param[in] socket_fd: socket句柄
 * @param[in] buf: 需要发送的数据
 * @param[in] len: 数据长度
 *
 * @returns:  -1 失败, 其它-实际发送的数据长度
 */
int joylink_softap_socket_send(int socket_fd, char *buf, int len);

/**
 * @brief: 释放softAP配网所使用的资源
 *
 * @return: 无返回值
*/
void  release_resource();


#ifdef _IS_DEV_REQUEST_ACTIVE_SUPPORTED_
/**
 * @name:joylink_softap_is_need_active 
 *
 * @returns:   
 */
extern int joylink_softap_is_need_active(void);
/**
 * @name:joylink_softap_active_clear 
 *
 * @returns:   
 */
extern int joylink_softap_active_clear(void);

#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
