

#ifdef _SOFTAP_SEND_BEACON_EN_
#include <string.h>
#include <time.h>

#include <pthread.h>
#include <unistd.h>

#include "joylink_log.h"

#include "joylink_softap_protocal.h"
#include "joylink_softap_beacon.h"
#include "joylink_softap_start.h"
// joylink platform layer header files
#include "joylink_stdio.h"
#include "joylink_string.h"
#include "joylink_memory.h"
#include "joylink_time.h"
#include "joylink_thread.h"
#include "joylink_log.h"
#include "joylink_softap.h"
#include "joylink_socket.h"

/**
 * @name:joylink_change_channel 
 *
 * @returns:   
 */

int32_t joylink_change_channel(char channel)
{
	return jl_wifi_change_channel(channel);
}

/**
 * @name:joylink_change_channel 
 *
 * @returns:   
 */

int joylink_wlan_send_mgnt(char *buffer, int len)
{
	jl_platform_print_buffer("send beacon:", buffer, len);

	return jl_wifi_80211_send(buffer, len);
}

/**
 * @name:joylink_monitor_loop_handler 
 *
 * @returns:   
 */

int joylink_beacon_packet(beacon_param_st *dev_param, char *packet_out, int max_len)
{
	char beacon_head[] = {
		0x80, 0x00, 0x3a, 0x01, 
		0xff, 0xff, 0xff, 0xff, 0xff, 0xff, // dest
		0x01, 0X02, 0x03, 0x04, 0x05, 0x06,  // source   mac
		0x01, 0x02, 0x03, 0x04, 0x05, 0x06,  // bssid    mac
		0xd0, 0x03, 
		0xe8, 0x61, 0x3b, 0X00, 0x00, 0x00, 
		0x00, 0x00, 0x64, 0x00, 0x21, 0x00};
	char beacon_end[] = {
		0x05, 0x04, 0x00, 0x01, 0x00, 0x00, 0x2a, 0x01, 0x00, 0x32, 0x04, 0x30, 0X48, 0x60, 0x6c, 0x2d, 
		0x1a, 0x20, 0x00, 0x01, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0X00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3d, 0X16, 0x01, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0X00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0xdd, 0x18, 0x00, 0x50, 0xf2, 0x02, 0x01, 0x01, 0x83, 0X00, 0x03, 0xa4, 0x00, 
		0x00, 0x27, 0xa4, 0x00, 0x00, 0x42, 0x43, 0x5e, 0x00, 0x62, 0x32, 0x2f, 0X00, 0xdd, 0x0e, 0x00, 
		0x50, 0xf2, 0x04, 0x10, 0x4a, 0x00, 0x01, 0x20, 0x10, 0x44, 0x00, 0x01, 0X02, 0xdd, 0x06, 0x00, 
		0xe0, 0x4c, 0x02, 0x01, 0x10};
	char beacon_rates[10] = {0x01, 0x08, 0x82, 0x84, 0x8b, 0x96,  0x0c, 0x12, 0x18, 0X24};
	
	char beacon_ssid[34] = {0};
	char beacon_dsps[3] = {0};

	int offset;
	int all_len;

	all_len = sizeof(beacon_head) + sizeof(beacon_end) + 10 + 34 + 3;

	if(dev_param == NULL || packet_out == NULL || max_len < all_len){
		return 0;
	}

	jl_platform_memcpy(beacon_head+10, dev_param->source, 6);
	jl_platform_memcpy(beacon_head+10+6, dev_param->Bssid, 6);

	beacon_ssid[0] = PACKET_80211_BEACON_TYPE_SSID;
	beacon_ssid[1] = jl_platform_strlen(dev_param->ssid);
	jl_platform_memcpy(beacon_ssid+2, dev_param->ssid, jl_platform_strlen(dev_param->ssid));

	beacon_dsps[0] = PACKET_80211_BEACON_TYPE_DSPS;
	beacon_dsps[1] = 1;
	beacon_dsps[2] = dev_param->channel;

	offset = 0;
	jl_platform_memcpy(packet_out, beacon_head, sizeof(beacon_head));
	offset += sizeof(beacon_head);
	jl_platform_memcpy(packet_out + offset, beacon_ssid, (jl_platform_strlen(dev_param->ssid)+2));
	offset += (jl_platform_strlen(dev_param->ssid)+2);
	jl_platform_memcpy(packet_out + offset, beacon_rates, sizeof(beacon_rates));
	offset += sizeof(beacon_rates);
	jl_platform_memcpy(packet_out + offset, beacon_dsps, sizeof(beacon_dsps));
	offset += sizeof(beacon_dsps);
	jl_platform_memcpy(packet_out + offset, beacon_end, sizeof(beacon_end));
	offset += sizeof(beacon_end);
	
	return offset;
}


/**
 * @name:joylink_softap_beacon_param_init 
 *
 * @returns:   
 */

// must to do

#define SOFTAP_DEFAULT_CHANNAL   6

char test_mac[6] = {0x01,0x02,0x03,0x04,0x05,0x06};

int joylink_softap_beacon_param_init(beacon_param_st *dev_param)
{
	if(dev_param == NULL){
		return 0;
	}
	jl_platform_memcpy(dev_param->source, test_mac, sizeof(test_mac));
	jl_platform_memcpy(dev_param->Bssid, test_mac, sizeof(test_mac));
	
	jl_platform_memcpy(dev_param->ssid, softap_ssid, jl_platform_strlen(softap_ssid));
	dev_param->channel = SOFTAP_DEFAULT_CHANNAL;

	return 0;
}

void joylink_softap_beacon_tast_exit(jl_thread_t *thread_handle)
{
	jl_platform_thread_exit(thread_handle);
}

/**
 * @name:joylink_monitor_loop_handler 
 *
 * @returns:   
 */

void joylink_softap_beacon_start_send(void)
{
	int i = 0;
	char ch_seq[] = {1,6,11,2,7,12,3,8,13,4,9,10,5};

	char tx_buff[1400] = {0};
	int tx_len;

	beacon_param_st dev_param;

	jl_platform_memset(&dev_param, 0, sizeof(dev_param));
	joylink_softap_beacon_param_init(&dev_param);

	tx_len = joylink_beacon_packet(&dev_param, tx_buff, 1400);
	
	while(1){
		joylink_change_channel(ch_seq[i++]);
		if(i >= sizeof(ch_seq))	{
			i=0;
		}
		
		joylink_wlan_send_mgnt(tx_buff, tx_len);
		jl_platform_msleep(200);

		joylink_change_channel(SOFTAP_DEFAULT_CHANNAL);
		jl_platform_msleep(130);

		if(joylink_softap_client_online()){
			break;
		}
    }

	joylink_softap_beacon_exit();
}

int joylink_softap_send_beacon_task(void)
{
	jl_thread_t thread_handle;
	
	jl_platform_memset(&thread_handle, 0, sizeof(jl_thread_t));
	
	thread_handle.handle = NULL;
	thread_handle.stackSize = 0x2000;
	thread_handle.thread_task = joylink_softap_beacon_start_send();
	thread_handle.parameter = NULL;
	thread_handle.isRunning = 1;

	jl_platform_thread_start(&thread_handle);

	return 0;
}

#endif //#ifdef _SOFTAP_SEND_BEACON_EN_


