#include "stdio.h"
#include <unistd.h>
// #include <sys/ioctl.h>
// #include <sys/socket.h>
// #include <netinet/in.h>
// #include <string.h>
// #include <arpa/inet.h>
#include <time.h>
#include "joylink_softap_protocal.h"
#include "joylink_softap_start.h"
#ifdef _SOFTAP_SEND_BEACON_EN_
#include "joylink_softap_beacon.h"
#endif
// joylink platform layer header files
#include "joylink_stdio.h"
#include "joylink_string.h"
#include "joylink_memory.h"
#include "joylink_time.h"
#include "joylink_thread.h"
#include "joylink_log.h"
#include "joylink_softap.h"
#include "joylink_socket.h"
#include "joylink_stdint.h"

#define JOYLINK_LOCAL_UDP_PORT 4320
#define JOYLINK_REMOTE_UDP_PORT 9999

#define JOYLINK_TCP_PORT 3000

#define JOYLINK_UDP_BROAD_IP  "255.255.255.255"

static uint8_t g_joylink_softap_status = 0;

/**
 * @name:joylink_udp_init 
 *
 * @param: port
 *
 * @returns:   
 */
static int joylink_udp_init(int port)
{
	int socket_fd = 0;

	jl_sockaddr_in sin; // struct sockaddr_in sin;
	// jl_sockaddr_in sin_recv; // struct sockaddr_in sin_recv;

	// socklen_t sin_len = sizeof(sin_recv);

    jl_platform_memset(&sin, 0, sizeof(sin));

	// struct timeval  selectTimeOut;
	// static uint32_t serverTimer;
	// static int interval = 0;

	sin.sin_family = jl_platform_get_socket_proto_domain(JL_SOCK_PROTO_DOMAIN_AF_INET);
	sin.sin_addr.s_addr = jl_platform_htonl(JL_INADDR_ANY);
	sin.sin_port = jl_platform_htons((unsigned short)port);

	socket_fd = jl_platform_socket(JL_SOCK_PROTO_DOMAIN_AF_INET, JL_SOCK_PROTO_TYPE_SOCK_DGRAM, JL_SOCK_PROTO_PROTO_IPPROTO_IP);

	int broadcastEnable = 1;
	if (jl_platform_setsockopt(socket_fd, 
                               JL_SOCK_OPT_LEVEL_SOL_SOCKET, 
                               JL_SOCK_OPT_NAME_SO_BROADCAST, 
                               (uint8_t *)&broadcastEnable, sizeof(broadcastEnable)) < 0){
		log_error("error: udp set broadcast error!\n");
		jl_platform_close(socket_fd);
		return -1;
	}

	// if(0 > bind(socket_fd, (struct sockaddr *)&sin, sizeof(struct sockaddr))){
	if(0 > jl_platform_bind(socket_fd, (jl_sockaddr *)&sin, sizeof(jl_sockaddr))){
		log_error("error: udp bind socket error!\n");
		jl_platform_close(socket_fd);
		return -1;
	}
	log_info("udp init is ok\n");

	return socket_fd;
}

/**
 * @name:joylink_tcp_init 
 *
 * @param: port
 *
 * @returns:   
 */
static int joylink_tcp_init(int port)
{
	int socket_fd; // ,new_fd;

	jl_sockaddr_in sin; // struct sockaddr_in sin;

	jl_platform_memset(&sin, 0, sizeof(sin));

	sin.sin_family = jl_platform_get_socket_proto_domain(JL_SOCK_PROTO_DOMAIN_AF_INET);
	sin.sin_addr.s_addr = jl_platform_htonl(JL_INADDR_ANY);
	sin.sin_port = jl_platform_htons((unsigned short)port);

	socket_fd = jl_platform_socket(JL_SOCK_PROTO_DOMAIN_AF_INET, JL_SOCK_PROTO_TYPE_SOCK_STREAM, JL_SOCK_PROTO_PROTO_IPPROTO_IP);

	// if(0 > bind(socket_fd, (struct sockaddr *)&sin, sizeof(struct sockaddr))){
	if(0 > jl_platform_bind(socket_fd, (jl_sockaddr *)&sin, sizeof(jl_sockaddr))){
		log_error("error: tcp bind socket error!\n");
		jl_platform_close(socket_fd);
		return -1;
	}

	if(0 > jl_platform_listen(socket_fd, 0))
	{
		log_error("error: tcp listen error!\n");
		jl_platform_close(socket_fd);
		return -1;
	}
	log_info("tcp init is ok\n");

	return socket_fd;
}

static int udp_fd = -1;
static int tcp_fd   = -1;
static int client_fd = -1;

int joylink_udp_broad_send(int socket_fd, char *buf, int len)
{
	int ret_len = 0;

	jl_sockaddr_in temp_addr; // struct sockaddr_in temp_addr;
	unsigned int temp_size = sizeof(temp_addr); // socklen_t temp_size = sizeof(temp_addr);

	if(socket_fd != udp_fd){
		jl_platform_printf("joylink udp broad send error!\n");		
		return -1;
	}
	
	jl_platform_memset(&temp_addr, 0, sizeof(temp_addr));

	temp_addr.sin_family = jl_platform_get_socket_proto_domain(JL_SOCK_PROTO_DOMAIN_AF_INET);
	temp_addr.sin_port = jl_platform_htons(JOYLINK_REMOTE_UDP_PORT);
	temp_addr.sin_addr.s_addr = jl_platform_inet_addr(JOYLINK_UDP_BROAD_IP);

	// ret_len = sendto(socket_fd, buf, len, 0, (struct sockaddr *)&temp_addr, temp_size);
	ret_len = jl_platform_sendto(socket_fd, buf, len, 0, (jl_sockaddr *)&temp_addr, temp_size, 0);
	log_info("joylink udp broad send data! fd: %d, len: %d\n", socket_fd, len);

	return ret_len;
}

jl_sockaddr_in jl_udp_recv;
unsigned int udp_size = sizeof(jl_udp_recv);

/**
 * @name:joylink_softap_socket_send 
 *
 * @param: socket_fd
 * @param: buf
 * @param: len
 *
 * @returns:   
 */
int joylink_softap_socket_send(int socket_fd, char *buf, int len)
{
	int ret_len = -1;

	if(socket_fd < 0 || buf == NULL || len <= 0){
		log_error("joylink socket send error!\n");
		return -1;
	}
	if(socket_fd == client_fd){
		ret_len = jl_platform_send(client_fd, buf, len, 0, 0);
		log_info("joylink tcp send len: %d ret: %d \n", len, ret_len);
	}else if(socket_fd == udp_fd){
		// ret_len = sendto(udp_fd, buf, len, 0, (struct sockaddr *)&jl_udp_recv, udp_size);
		ret_len = jl_platform_sendto(udp_fd, buf, len, 0, (jl_sockaddr *)&jl_udp_recv, udp_size, 0);
		log_info("joylink udp send len: %d ret: %d \n", len, ret_len);
	}else{
		log_error("joylink socket send fd error!\n");	
	}
	return ret_len;
}

static uint8_t client_online_flag = 0;

//#define _SOFTAP_SEND_BEACON_EN_

/**
 * @name:joylink_softap_start 
 *
 * @returns:   
 */
int joylink_softap_start(unsigned int timeout_ms)
{
	int ret_len = 0;
	char recBuffer[1024] = {0};
	int ret = 0;
	int i = 0;
	int data_ret = 0;
	int max_fd;
	jl_timeval time_out; // struct timeval time_out;
	uint32_t softap_time_start =  jl_platform_time_clock_ms();
	uint32_t softap_time_now;
	uint32_t time_start = jl_platform_time_clock_ms()/1000;
	uint32_t time_now;
    uint32_t time_gap = 0;

	joylinkSoftAP_Result_t softap_res;
	jl_fd_set readfds = jl_platform_fd_set_allocate();

	client_online_flag = 0;

#ifdef _SOFTAP_SEND_BEACON_EN_
	joylink_softap_send_beacon_task();
#endif

	jl_platform_memset(&softap_res,0,sizeof(joylinkSoftAP_Result_t));
	g_joylink_softap_status = 1;
	
	udp_fd = joylink_udp_init(JOYLINK_LOCAL_UDP_PORT);
	tcp_fd = joylink_tcp_init(JOYLINK_TCP_PORT);

	joylink_softap_init();

	while(1){
		max_fd = -1;
		JL_FD_ZERO(readfds);

		if(udp_fd >= 0){
			JL_FD_SET(udp_fd, readfds);
		}
		if(tcp_fd >= 0){
			JL_FD_SET(tcp_fd, readfds);
		}
		if(client_fd >= 0){
			JL_FD_SET(client_fd, readfds);
		}
		if(max_fd < udp_fd){
			max_fd = udp_fd;
		}
		if(max_fd < tcp_fd){
			max_fd = tcp_fd;
		}
		if(max_fd < client_fd){
			max_fd = client_fd;
		}

		time_out.tv_usec = 0;
		time_out.tv_sec = 2;

		time_now = jl_platform_time_clock_ms() / 1000;
        if (time_now >= time_start) {
            time_gap = (uint32_t) (time_now - time_start);
        } else {
            time_gap = (uint32_t) (4294967295 - time_start + time_now);
        }
		// if(udp_broad_disable == 0 && udp_fd >= 0 && (time_now - time_start) >= 2){
		if(client_online_flag == 0 && udp_fd >= 0 && time_gap >= 2){
			time_start = time_now;
			log_info("joylink softap udp broad\n");
			joylink_softap_udpbroad(udp_fd);
		}
		log_info("\njoylink wait data!\n\n");
        
		//net config timeout check
        softap_time_now = jl_platform_time_clock_ms();
        if (softap_time_now >= softap_time_start) {
            time_gap = (uint32_t) (softap_time_now - softap_time_start);
        } else {
            time_gap = (uint32_t) (4294967295 - softap_time_start + softap_time_now);
        }
		log_info("time_gap %d, timeout_ms %d\n", time_gap, timeout_ms);
		// if(jl_platform_time_clock_ms() - softap_time_start > timeout_ms) {
		if(time_gap > timeout_ms) {
			release_resource();
			//超时需要返回 -1
			ret = -1;
			break;
		} 

		ret = jl_platform_select(max_fd+1, readfds, NULL, NULL, &time_out);
		if (ret <= 0){
			continue;
		}
		for(i = 0; i < ret; i++){
			if(JL_FD_ISSET(tcp_fd, readfds)){
				int new_fd;
				jl_sockaddr_in client_addr; // struct sockaddr_in client_addr;
				uint32_t client_size = sizeof(client_addr);

				jl_platform_memset(&client_addr, 0, sizeof(client_addr));
				new_fd = jl_platform_accept(tcp_fd, (jl_sockaddr *)(&client_addr), &client_size);
				if(new_fd >= 0){
					if(client_fd < 0){
						client_fd = new_fd;
						log_error("accept a client! IP = %s:%d\n", 
						jl_platform_inet_ntoa(client_addr.sin_addr), client_addr.sin_port);
					}else{
						jl_platform_close(new_fd);
						log_info("error: tcp have a client Already!\n");
					}
					continue;
				}
			}
			if( (client_fd >= 0) && JL_FD_ISSET(client_fd, readfds) ){
				jl_platform_memset(recBuffer, 0, 1024);
				ret_len = jl_platform_recv(client_fd, recBuffer, 1024, 0, 0);
				if(ret_len <= 0)
				{ 
				 	jl_platform_close(client_fd);
					client_fd = -1;
					log_error("tcp recv data error!,tcp close\n");
					continue;
				}
				// log_info("tcp recv data: %s len: %d\n", recBuffer, ret_len);
				data_ret = joylink_softap_data_packet_handle(client_fd, (uint8_t *)recBuffer, ret_len);
				if(data_ret > 0){
					client_online_flag = 1;
				}else{
					client_online_flag = 0;
				}
			}
			if(JL_FD_ISSET(udp_fd, readfds)){
				jl_platform_memset(recBuffer, 0, 1024);
				// ret_len = recvfrom(udp_fd, recBuffer, sizeof(recBuffer), 0, (struct sockaddr *)&jl_udp_recv, &udp_size);
				ret_len = jl_platform_recvfrom(udp_fd, recBuffer, sizeof(recBuffer), 0, (jl_sockaddr *)&jl_udp_recv, (uint32_t *)&udp_size,  0);
				// ret_len = jl_platform_recvfrom(udp_fd, (jl_sockaddr *)&jl_udp_recv, recBuffer, sizeof(recBuffer), 200);
				if(ret_len < 0){
					jl_platform_close(udp_fd);
					udp_fd = -1;
					log_error("udp recv data error!,UDP close\n");
					continue;
				}
				// log_info("udp recv data: %s len: %d\n", recBuffer, ret_len);
				data_ret = joylink_softap_data_packet_handle(udp_fd, (uint8_t *)recBuffer, ret_len);
				if(data_ret > 0){
					client_online_flag = 1;
				}else{
					client_online_flag = 0;
				}
			}
		}
		if(joylink_softap_result(&softap_res) != -1){
			log_info("softap finsh,ssid->%s,pass->%s",softap_res.ssid,softap_res.pass);
			release_resource();
			/*
				need user to end 
			*/
            jl_softap_connect_wifi_router((char *)softap_res.ssid, (char *)softap_res.pass);
			ret = 0;
			break;
		}
		
	}
	jl_platform_fd_set_free(readfds);

	return ret;
}

/**
 * @brief: client online flag 
 *
 * @returns: status
 */
uint8_t joylink_softap_client_online(void)
{
	return client_online_flag;
}

/*
* @brief release  fd resource
*  @return none
*/
void  release_resource()
{
    if(udp_fd >= 0)
    {
        log_info("close fd udp_fd");
        jl_platform_close(udp_fd);
    }

    if(tcp_fd >= 0)
    {
        log_info("close fd tcp_fd");
        jl_platform_close(tcp_fd);
    }

    if(client_fd >= 0)
    {
        log_info("close fd client_fd");
        jl_platform_close(client_fd);
    }
	g_joylink_softap_status = 0;
}


/**
 * @name:joylink_softap_status 
 *
 * @returns:   0，不是softap配网状态；1，softap配网进行中
 */
uint8_t joylink_softap_status(void)
{
	return g_joylink_softap_status;
}
