#include <stdio.h>
#include <string.h>
// #include <fcntl.h>
#include <errno.h>

#include "joylink_utils.h"

// joylink platform layer header files
#include "joylink_stdio.h"
#include "joylink_string.h"
#include "joylink_memory.h"
#include "joylink_time.h"
#include "joylink_thread.h"
#include "joylink_log.h"
#include "joylink_softap.h"
#include "joylink_socket.h"
#include "joylink_stdint.h"
#include "joylink_ret_code.h"

/**
 * brief: 
 *
 * @Param: pBytes
 * @Param: srcLen
 * @Param: pDstStr
 * @Param: dstLen
 *
 * @Returns: 
 */
int 
joylink_util_byte2hexstr(const uint8_t *pBytes, int srcLen, uint8_t *pDstStr, int dstLen)
{
	const char tab[] = "0123456789abcdef";
	int i = 0;

	jl_platform_memset(pDstStr, 0, dstLen);

	if(dstLen < srcLen * 2){
		srcLen = (dstLen - 1) / 2;
    }

	for(i = 0; i < srcLen; i++){
		*pDstStr++ = tab[*pBytes >> 4];
		*pDstStr++ = tab[*pBytes & 0x0f];
		pBytes++;
	}
	*pDstStr++ = 0;

	return srcLen * 2;
}

/**
 * brief: 
 *
 * @Param: hexStr
 * @Param: buf
 * @Param: bufLen
 *
 * @Returns: 
 */
int 
joylink_util_hexStr2bytes(const char *hexStr, uint8_t *buf, int bufLen)
{
	int i;
	int len;

	if(NULL == hexStr){
		len = 0;
	}else{
		len = (int)jl_platform_strlen(hexStr) / 2;
		if(bufLen < len){
			len = bufLen;
        }
	}
	jl_platform_memset(buf, 0, bufLen);

	for(i = 0; i < len; i++){
		char ch1, ch2;
		int val;

		ch1 = hexStr[i * 2];
		ch2 = hexStr[i * 2 + 1];
		if(ch1 >= '0' && ch1 <= '9'){
			val = (ch1 - '0') * 16;
        }else if (ch1 >= 'a' && ch1 <= 'f'){
			val = ((ch1 - 'a') + 10) * 16;
        }else if (ch1 >= 'A' && ch1 <= 'F'){
			val = ((ch1 - 'A') + 10) * 16;
        }else{
			return -1;
        }

		if(ch2 >= '0' && ch2 <= '9'){
			val += ch2 - '0';
        }else if (ch2 >= 'a' && ch2 <= 'f'){
			val += (ch2 - 'a') + 10;
        }else if (ch2 >= 'A' && ch2 <= 'F'){
			val += (ch2 - 'A') + 10;
        }else{
			return -1;
        }

		buf[i] = val & 0xff;
	}

	return 0;
}

/**
 * brief: 
 *
 * @Param: pPeerAddr
 * @Param: str
 *
 * @Returns: 
 */
int 
// joylink_util_get_ipstr(struct sockaddr_in* pPeerAddr, char* str)
joylink_util_get_ipstr(jl_sockaddr_in* pPeerAddr, char* str)
{
	jl_sockaddr_in* pAddr = (jl_sockaddr_in*)pPeerAddr;

	int n = jl_platform_sprintf(str, "%s:%d",
		(char *)jl_platform_inet_ntoa(pAddr->sin_addr),
		pAddr->sin_port
		);

	return n;
}

/**
 * brief: 
 *
 * @Param: ipport
 * @Param: out_ip
 * @Param: out_port
 *
 * @Returns: 
 */
int 
joylink_util_cut_ip_port(const char *ipport, char *out_ip, int *out_port)
{
    if(NULL == ipport || 
            NULL == out_ip ||
            NULL == out_port){
        return E_RET_ERROR;
    }
    int ret = E_RET_OK;
    int offset = 0;
    int len = jl_platform_strlen(ipport);

    while( (offset < len) && (ipport[offset] != ':')){
        offset++;
    }
    
    if(offset < len){
        jl_platform_memcpy(out_ip, ipport, offset);
        out_ip[offset] = '\0';
        *out_port = jl_platform_atoi(ipport + offset + 1);
    }else{
        ret = E_RET_ERROR;
    }

	return ret;
}

/**
 * brief: 
 *
 * @Param: msg
 * @Param: is_fmt
 * @Param: num_line
 * @Param: buff
 * @Param: len
 */
void
joylink_util_print_buffer(const char *msg, int is_fmt, int num_line, const uint8_t *buff, int len)
{
    if(NULL == msg || NULL == buff){
        return;
    }
    int cut_num = 4;
    int i = 0;
    jl_platform_printf("%s\n", msg);
    for(i =0; i < len; i++){
        jl_platform_printf("%02x ", (int)buff[i]);
        if(is_fmt && !(num_line%cut_num)){
            if(!((i + 1)%cut_num)){
                jl_platform_printf("| ");
            }
        }

        if(!((i + 1)%num_line)){
            jl_platform_printf("\n");
        }
    }
    jl_platform_printf("\n");
}



/**
 * [joylink_util_malloc]
 *
 * @param: [size]
 *
 * @returns:
 */
void *joylink_util_malloc(size_t size)
{

    void *p = jl_platform_malloc(size);
    if (!p){
        return p;
    }
    jl_platform_memset(p, 0, size);

    return p;
}

/**
 * [joylink_util_free]
 *
 * @param: [p]
 */
void joylink_util_free(void *p)
{
    if(p != NULL){
        jl_platform_free(p);
        // p = NULL;
    }
}

/**
 * [joylink_util_freep]
 *
 * @param: [p]
 */
void joylink_util_freep(void **p)
{
    if (p != NULL && *p != NULL) {
        joylink_util_free(*p);
        *p = NULL;
    }
}

/**
 * brief: add TLV data to buf
 * @Param: buf,buffer address
 * @Param: tag,
 * @Param: lc,tag length
 * @Param: value,tag value
 *
 * @Returns:0-failed,>0,added length
 */
uint8_t joyTLVDataAdd(uint8_t *buf,uint8_t tag, uint8_t lc, uint8_t *value)
{
    uint8_t lenthtotal = 0;
    if(buf == NULL || value == NULL){
        return 0;
    }
    *buf        = tag;
    *(buf + 1)  = lc;
    jl_platform_memcpy(buf + 2, value, lc);
    lenthtotal = lc + 2;
    return lenthtotal;
}

/**
 * brief:
 *
 * @Param: src
 * @Param: dst
 * @Param: len
 *
 * @Returns:
 */
int joylink_util_reverse_buf(const uint8_t *src, uint8_t *dst, int32_t len)
{
    jl_platform_memset(dst, 0x0, len);

    int32_t i = 0;

    for (i = 0;i < len;i++)
    {
        dst[i] = src[len - 1 - i];
    }
    return 0;
}
