#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// joylink platform layer header files
#include "joylink_stdio.h"
#include "joylink_string.h"
#include "joylink_flash.h"

/**
 * @brief:set data in jl_flash
 *
 * @param: key, value
 *
 * @returns: success 0, failed -1
 */
int32_t jl_flash_SetInt8(const char* key, int8_t value)
{
#ifdef __LINUX_PAL__
    FILE *fp = NULL;

    fp = jl_platform_fopen(key, "wb+");
    if (fp == NULL) {
        jl_platform_printf("[SetInt8] Error, key = %s\r\n", key);
        return -1;
    }

    jl_platform_fwrite(&value, 1, sizeof(int8_t), fp);
    jl_platform_fclose(fp);
    return 0;
#else
    return 0;
#endif
}

int32_t jl_flash_SetInt32(const char* key, int32_t value)
{
#ifdef __LINUX_PAL__
    FILE *fp = NULL;

    fp = jl_platform_fopen(key, "wb+");
    if (fp == NULL) {
        jl_platform_printf("[SetInt32] Error, key = %s\r\n", key);
        return -1;
    }
    jl_platform_fwrite(&value, 1, sizeof(int32_t), fp);
    jl_platform_fclose(fp);
    return 0;
#else
    return 0;
#endif
}

int32_t jl_flash_SetString(const char* key, const char* value)
{
#ifdef __LINUX_PAL__
    int32_t ret = -1, length;
    FILE *fp = NULL;

    length = jl_platform_strlen(value);
    fp = jl_platform_fopen(key, "wb+");
    if (fp == NULL) {
        jl_platform_printf("[SetString] Error, key = %s\r\n", key);
        return -1;
    }
    ret = jl_platform_fwrite(value, 1, length, fp);
    if (ret <= 0 || ret != length) {
        printf("[SetString] jl_platform_fwrite error!\n");
        ret = -1;
    } else {
        printf("[SetString] jl_platform_fwrite success, ret:%d\n", ret);
        ret = 0;
    }
    jl_platform_fclose(fp);
    return ret;
#else
    return 0;
#endif
}

int32_t jl_flash_SetBool(const char* key, int8_t value)
{
#ifdef __LINUX_PAL__
    FILE *fp = NULL;

    fp = jl_platform_fopen(key, "wb+");
    if (fp == NULL) {
        jl_platform_printf("[SetBool] Error, key = %s\r\n", key);
        return -1;
    }
    jl_platform_fwrite(&value, 1, sizeof(int8_t), fp);
    jl_platform_fclose(fp);
    return 0;
#else
    return 0;
#endif
}


/**
 * @brief:get data from jl_flash
 *
 * @param: key, value
 *
 * @returns: 读取到的value值或value长度len, 如果读取失败则返回-1
 */
int8_t 	jl_flash_GetInt8(const char* key, int8_t defaultValue)
{
#ifdef __LINUX_PAL__
    FILE *fp = NULL;
    int8_t value = 0;

    fp = jl_platform_fopen(key, "rb");
    if (fp == NULL) {
        jl_platform_printf("[GetInt8] Error, key = %s\r\n", key);
        return -1;
    }
    if (jl_platform_fread(&value, 1, sizeof(int8_t), fp) <= 0) {
        value = defaultValue;
        printf("[GetInt8] jl_platform_fread failed, value = defaultValue\n");
    }
    jl_platform_fclose(fp);
    printf("[GetInt8] value = %d\n", value);
    return value;
#else
    return 0;
#endif
}

int32_t jl_flash_GetInt32(const char* key, int32_t defaultValue)
{
#ifdef __LINUX_PAL__
    FILE *fp = NULL;
    int32_t value = 0;

    fp = jl_platform_fopen(key, "rb");
    if (fp == NULL) {
        jl_platform_printf("[GetInt32] Error, key = %s\r\n", key);
        return -1;
    }
    if (jl_platform_fread(&value, 1, sizeof(int32_t), fp) != 4) {
        value = defaultValue;
        printf("[GetInt32], jl_platform_fread failed, value = defaultValue\n");
    }
    jl_platform_fclose(fp);
    printf("[GetInt32] value = %d\n", value);
    return value;
#else
    return 0;
#endif
}

int32_t jl_flash_GetString(const char* key, char* value)
{
#ifdef __LINUX_PAL__
    int32_t read_len, ret;
    FILE *fp = NULL;

    fp = jl_platform_fopen(key, "rb");
    if (fp == NULL) {
        jl_platform_printf("[GetString] Error, key = %s\r\n", key);
        return -1;
    }

    read_len = 0;
    while (1)
    {
        ret = jl_platform_fread(value + read_len, 1, 0x200, fp);
        if (ret <= 0)
            break;
        read_len += ret;
    }

    jl_platform_fclose(fp);
    return read_len;
#else
    return 0;
#endif
}

int8_t  jl_flash_GetBool(const char* key, int8_t defaultValue)
{
#ifdef __LINUX_PAL__
    FILE *fp = NULL;
    int8_t value = 0;

    fp = jl_platform_fopen(key, "rb");
    if (fp == NULL) {
        jl_platform_printf("[GetBool] Error, key = %s\r\n", key);
        return -1;
    }
    if (jl_platform_fread(&value, 1, sizeof(int8_t), fp) == 0) {
        value = defaultValue;
        printf("[GetBool] jl_platform_fread failed, value = defaultValue\n");
    }
    jl_platform_fclose(fp);
    fp = NULL;
    printf("[GetBool] value = %d\n", value);
    return value;
#else
    return 0;
#endif
}

/**
 * @brief:clear jl_flash data 
 *
 * @param: key
 *
 * @returns: none
 */
void jl_flash_ClearKey(const char *key)
{
#ifdef __LINUX_PAL__
    char buf[30];
    jl_platform_sprintf(buf, "rm -rf %s", key);
    system(buf);
#else
    return 0;
#endif
}

/**
 * @brief: 二进制数据写入flash
 * 
 * @param[in] key: 关键字
 * 
 * @param[in] offset: flash地址偏移
 * 
 * @param[in] buf: 写入flash的数据
 * 
 * @param[in] len: 要写入数据的长度
 * 
 * @return 返回实际写入的数据长度，写入失败则返回-1
 * 
 */

int32_t jl_flash_SetBuffer(const char* key, const void* buf, int32_t len)
{
#ifdef __LINUX_PAL__
    int32_t ret;
    FILE *fp = NULL;

    fp = jl_platform_fopen(key, "wb+");
    if (fp == NULL) {
        jl_platform_printf("[SetBuffer] Error, key = %s\r\n", key);
        return -1;
    }
    ret = jl_platform_fwrite(buf, 1, len, fp);

    jl_platform_fclose(fp);
    return ret;
#else
    return 0;
#endif
}

/**
 * @brief: 读取指定偏移地址的数据
 * 
 * @param[in] key: 关键字
 * 
 * @param[in] offset: flash偏移地址
 * 
 * @param[out] buf: 存储读取数据的buffer
 * 
 * @param[in] len: 预期读取的数据长度
 * 
 * @return int32_t: 实际读取的数据长度，如果读取失败则返回-1
 * 
 */ 

int32_t jl_flash_GetBuffer(const char* key, const void* buf, int32_t len)
{
#ifdef __LINUX_PAL__
    int32_t ret;
    FILE *fp = NULL;

    fp = jl_platform_fopen(key, "rb");
    if (fp == NULL) {
        jl_platform_printf("[GetBuffer] Error, key = %s\r\n", key);
        return -1;
    }
    ret = jl_platform_fread(buf, 1, len, fp);

    jl_platform_fclose(fp);
    return ret;
#else
    return 0;
#endif
}
